# This has nothing to do with the library, these are just
# Some informations you need to know

def about():
    return("GEOMATH is a Library developed for handling\
    Analytical Geometry on Python 3.X")

def version():
    return("Version: 0.1.2.3")



#Figure file for the GeoMath library
from math import sqrt

class Figure:
    "Figure Object in GeoMath library"

    def __init__(self, figureType=None):
        self.polygonPoints = []

    def add_point(self, point):
        self.polygonPoints.append((point))

    def add_points(self, point):
        for p in point:
            self.polygonPoints.append((p))

    def area(self):
        """
        Area Class based on a StackOverflow post
        """
        n = len(self.polygonPoints) # of corners
        a = 0.0
        for i in range(n):
            j = (i + 1) % n
            a += abs(self.polygonPoints[i].x * self.polygonPoints[j].y - self.polygonPoints[j].x * self.polygonPoints[i].y)
        result = a / 2.0
        return result

    def perimeter(self):
        """
        Perimeter Property for Figure Object in GeoMath library
        """
        n = len(self.polygonPoints)
        p = 0.0
        for i in range(n):
            j = (i + 1) % n
            p += abs(self.polygonPoints[i].distance(self.polygonPoints[j]))
        return(p)

    def barycenter(points):
        """
        Barycenter Property for Figure Object in GeoMath library
        """
        i = 0
        arrayx = []
        arrayy = []
        for point in points:
            arrayx.append(point.x)
            arrayy.append(point.y)
        XPoint = sum(arrayx) / len(arrayx)
        YPoint = sum(arrayx) / len(arrayy)
        return(("Point(%s, %s)") % (XPoint, YPoint))

    def __repr__(self):
        points = ""
        for point in self.polygonPoints:
            pontos = pontos + str(point) + " "
        figureString = "Figure Points: " + points
        return(figureString)

class Circumference():
    def __init__(self):
        self.centerPoint = None
        self.outsidePoint = None
        self.Eq = None
        self.Rad = 0
        self.A = 0
        self.Per = 0
        self.createdby = None

    def create_points(self, centerPoint, outsidePoint):
        from .point import Point
        import math
        self.centerPoint = centerPoint
        self.outsidePoint = outsidePoint
        self.Rad = centerPoint.distance(outsidePoint)
        self.createdby = "points"
        self.Eq = "({}-{})ˆ2 + ({}-{})^2={}ˆ2".format(outsidePoint.x, centerPoint.x, outsidePoint.y, centerPoint.y, self.Rad)
        self.A = float(self.Rad) ** float(2) * math.pi #math.pi * (float(self.radius) ** float(2))
        self.Per = 2 * math.pi * self.Rad

    def create_equation(self, equation):
        self.Eq = equation
        self.createdby = "equation"

    def area(self):
        return(self.A)

    def perimeter(self):
        return(self.Per)

    def equation(self):
        if self.createdby == "equation":
            return(self.Eq)
        return(self.Eq)

    def radius(self):
        return(self.Rad)



# Point file for the GeoMath library
from math import sqrt
class Point:
    "Point Object in GeoMath library"

    # Defines the Point(x, y)
    def __init__(self, x, y):
        self.x = x
        self.y = y

    # Midpoint between "self" and "PointTwo".
    def midpoint(self, PointTwo):
        '''
        :param: PointTwo - Point subclass
        :return: Point subclass
        '''
        Xm = ((self.x + PointTwo.x)/2)
        Ym = ((self.y + PointTwo.y)/2)

        return(Point(Xm, Ym))

    # Euclidean distance between "self" and "PointTwo".
    def distance(self, PointTwo=None):
        '''
        If other is not specified, the origin point is used.

        :param: other - Point subclass
        :return: float
        '''
        if PointTwo is None:
            PointTwo = self.__class__()

        Xd = pow(PointTwo.x - self.x, 2)
        Yd = pow(PointTwo.y - self.y, 2)

        return(sqrt(Xd + Yd))

    # The quadrant of point  "self".
    def quadrant(self):
        if self.x > 0 and self.y > 0:
            return(1)
        elif self.x < 0 and self.y > 0:
            return(2)
        elif self.x < 0 and self.y < 0:
            return(3)
        elif self.x > 0 and self.y < 0:
            return(4)
        else:
            return(0)

    # Defines the output of the point
    def __repr__(self):
        return(("%s(%s, %s)") % (self.__class__.__name__, self.x, self.y))



from distutils.core import setup
setup(
  name = 'geomath',
  packages = ['geomath'],
  version = '0.1.2.3',
  description = 'A Mathematic Library to Handle Eucliedan Geometry',
  author = 'Vinicius Mesel',
  author_email = 'me@vmesel.com',
  url = 'https://github.com/vmesel/GeoMath/',
  download_url = 'https://www.github.com/vmesel/GeoMath/tarball/0.1.2.3',
  keywords = ['geometry', 'math', 'euclidean geometry'],
  classifiers = [],
)



#Line file for the GeoMath library
from math import sqrt, atan, degrees
from geomath import point as p
import re

class Line:
    def __init__(self):
        """
        These are the initial line parameters
        """
        self.PointOne = 0
        self.PointTwo = 0
        self.Angulation = 0
        self.Angular = 0
        self.Linear = 0
        self.A = 0
        self.B = 0
        self.C = 0
        self.createdby = None

    def create(self,PointOne, PointTwo):
        """
        This function is for getting a line created by two points
        """
        self.PointOne = PointOne
        self.PointTwo = PointTwo
        self.createdby = "points"
        self.A = float(self.PointOne.y - self.PointTwo.y)
        self.B = float(self.PointTwo.x - self.PointOne.x)
        if self.B > 0 or self.B == 0:
            self.B = "+" + str(self.B)
        self.C = ((self.PointOne.x * self.PointTwo.y) - (self.PointOne.y * self.PointTwo.x))
        if self.C > 0 or self.C == 0:
            self.C = "+" + str(self.C)

    def create_via_equation(self, equation):
        """
        This function is for getting a line created by a line equation
        """
        self.createdby = "equation"
        regx = re.compile(r'(\W?\d+)\S(\W\d+)\S(\W\d+)\=(\W?\d+)')
        values = regx.findall(equation)
        values = values[0]
        self.A = values[0]
        self.B = values[1]
        self.C = values[2]
        self.Angular = (float(values[0]) / float(values[1]))
        self.Angulation = degrees(atan(float(self.Angular)))
        self.PointOne = p.Point(float(self.C) / float(self.A),0)
        self.PointTwo = p.Point(0, float(self.C) / float(self.B))


    def equation(self):
        return("{}x{}y{}=0".format(self.A, self.B, self.C))

    def get_angle(self):
        return(self.Angulation)

    def __repr__(self):
        if self.createdby == "equation":
            return("A: {} B: {} C: {}".format(self.A, self.B, self.C))
        elif self.createdby == "points":
            return("Line({},{})".format(self.PointOne, self.PointTwo))

    # HERE THE LINE ONLY ATTRIBUTES END

    def point_distance(self, PointTwo):
        EquationA = ((float(self.A) * float(PointTwo.x)) + (float(self.B) * float(PointTwo.y)) + float(self.C))
        if EquationA == 0:
            raise ValueError('Point is inside the line!')
        EquationB = sqrt((float(self.A) * float(self.A)) + (float(self.B) * float(self.B)))
        return((float(EquationA) / float(EquationB)))


    def comparisor(self, LineTwo):
        angLineTwo = LineTwo.Angular
        if self.Angular == angLineTwo:
            return("Paralell Lines")
        elif self.Angular is not angLineTwo:
            return("Function under development")
            #We need to detect the other types of lines

    def point_alignment(self, PointThree):
        #Function under development
        pass
